@@ -94,11 +94,13 @@ void pkgAcqMethod::Fail(string Err,bool Transient)
       if (*I == '\n') 
 	 *I = ' ';
    }
-
+   
+   char S[1024];
+   char *End = S;
    if (Queue != 0)
    {
-      std::cout << "400 URI Failure\nURI: " << Queue->Uri << "\n"
-                << "Message: " << Err << " " << IP << "\n";
+      End += snprintf(S,sizeof(S)-50,"400 URI Failure\nURI: %s\n"
+		      "Message: %s %s\n",Queue->Uri.c_str(), Err.c_str(), IP.c_str());
       // Dequeue
       FetchItem *Tmp = Queue;
       Queue = Queue->Next;
@@ -107,17 +109,22 @@ void pkgAcqMethod::Fail(string Err,bool Transient)
 	 QueueBack = Queue;
    }
    else
-      std::cout << "400 URI Failure\nURI: <UNKNOWN>\nMessage: " << Err << "\n";
-
+   {
+      End += snprintf(S,sizeof(S)-50,"400 URI Failure\nURI: <UNKNOWN>\n"
+		      "Message: %s\n",Err.c_str());
+   }
    if(FailReason.empty() == false)
-      std::cout << "FailReason: " << FailReason << "\n";
+      End += snprintf(End,sizeof(S)-50 - (End - S),"FailReason: %s\n",FailReason.c_str());
    if (UsedMirror.empty() == false)
-      std::cout << "UsedMirror: " << UsedMirror << "\n";
-   // Set the transient flag
+      End += snprintf(End,sizeof(S)-50 - (End - S),"UsedMirror: %s\n",UsedMirror.c_str());
+   // Set the transient flag 
    if (Transient == true)
-      std::cout << "Transient-Failure: true\n";
-
-   std::cout << "\n" << std::flush;
+      strcat(S,"Transient-Failure: true\n\n");
+   else
+      strcat(S,"\n");
+   
+   if (write(STDOUT_FILENO,S,strlen(S)) != (signed)strlen(S))
+      exit(100);
 }
 									/*}}}*/
 // AcqMethod::URIStart - Indicate a download is starting		/*{{{*/
@@ -142,6 +149,8 @@ void pkgAcqMethod::URIStart(FetchResult &Res)
    if (Res.ResumePoint != 0)
       End += snprintf(End,sizeof(S)-4 - (End - S),"Resume-Point: %lu\n",
 		      Res.ResumePoint);
+   if (UsedMirror.empty() == false)
+      End += snprintf(End,sizeof(S)-4 - (End - S),"UsedMirror: %s\n",UsedMirror.c_str());
       
    strcat(End,"\n");
    if (write(STDOUT_FILENO,S,strlen(S)) != (signed)strlen(S))
@@ -371,9 +380,10 @@ int pkgAcqMethod::Run(bool Single)
 	    
 	    Tmp->Uri = LookupTag(Message,"URI");
 	    Tmp->DestFile = LookupTag(Message,"FileName");
-	    if (StrToTime(LookupTag(Message,"Last-Modified"),Tmp->LastModified) == false)
+	    if (RFC1123StrToTime(LookupTag(Message,"Last-Modified").c_str(),Tmp->LastModified) == false)
 	       Tmp->LastModified = 0;
 	    Tmp->IndexFile = StringToBool(LookupTag(Message,"Index-File"),false);
+	    Tmp->FailIgnore = StringToBool(LookupTag(Message,"Fail-Ignore"),false);
 	    Tmp->Next = 0;
 	    
 	    // Append it to the list
@@ -408,11 +418,18 @@ void pkgAcqMethod::Log(const char *Format,...)
    va_list args;
    va_start(args,Format);
 
-   // fprintf the description
-   fprintf(stdout,"101 Log\nURI: %s\n"
-			       "Message: ",CurrentURI.c_str());
-   vfprintf(stdout,Format,args);
-   std::cout << "\n\n" << std::flush;
+   // sprintf the description
+   char S[1024];
+   unsigned int Len = snprintf(S,sizeof(S)-4,"101 Log\n"
+                               "URI: %s\n"
+                               "UsedMirror: %s\n"
+			       "Message: ", UsedMirror.c_str(),
+                               CurrentURI.c_str());
+   vsnprintf(S+Len,sizeof(S)-4-Len,Format,args);
+   strcat(S,"\n\n");
+   
+   if (write(STDOUT_FILENO,S,strlen(S)) != (signed)strlen(S))
+      exit(100);
 }
 									/*}}}*/
 // AcqMethod::Status - Send a status message				/*{{{*/
@@ -427,11 +444,19 @@ void pkgAcqMethod::Status(const char *Format,...)
    va_list args;
    va_start(args,Format);
 
-   // fprintf the description
-   fprintf(stdout, "102 Status\nURI: %s\n"
-           "Message: ",CurrentURI.c_str());
-   vfprintf(stdout,Format,args);
-   std::cout << "\n\n" << std::flush;
+   // sprintf the description
+   char S[1024];
+   unsigned int Len = snprintf(S,sizeof(S)-4,"102 Status\n"
+                               "URI: %s\n"
+                               "UsedMirror: %s\n"
+			       "Message: ",UsedMirror.c_str(),
+                               CurrentURI.c_str());
+
+   vsnprintf(S+Len,sizeof(S)-4-Len,Format,args);
+   strcat(S,"\n\n");
+   
+   if (write(STDOUT_FILENO,S,strlen(S)) != (signed)strlen(S))
+      exit(100);
 }
 									/*}}}*/
 // AcqMethod::Redirect - Send a redirect message                       /*{{{*/
